@@ -1,4 +1,4 @@
-from .password_hash import password_hash, PasswordHashAlgorithm
+from .password_hash import password_hash, PasswordHashAlgorithm, password_digest_hash
 from .rfc2617 import digest_client, digest_server
 from .dbuser import BaseDBUser
 from .signed_token_jwt import SignedToken, SignedTokenExpiredError, SignedTokenImmatureError, \
@@ -7,3 +7,4 @@ from .token_box import TokenBox
 from .token_lib import TokenLib
 from .authenticators import authenticator
 from .auth_client import TokenAuthClientMixin, AuthenticationError
+
